<?php
namespace Followize\Extension\CF7;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

App::uses( 'contact-form-7', 'View' );

class ContactForm7_Controller
{
	public function __construct()
	{
		add_filter( 'wpcf7_editor_panels', array( &$this, 'show_panel' ) );
		add_filter( 'wpcf7_pre_construct_contact_form_properties', array( &$this, 'insert_property' ) );
		add_action( 'wpcf7_save_contact_form', array( &$this, 'save_property' ) );
		add_action( 'wpcf7_before_send_mail', array( &$this, 'do_integration' ) );
		add_filter( 'wpcf7_form_hidden_fields', array( &$this, 'add_hub_utmz_hidden' ) );
	}

	public function show_panel( $panels )
	{
		$panels['followize-extension'] = array(
			'title'    => __( 'Followize', App::PLUGIN_SLUG ),
			'callback' => array( __NAMESPACE__ . '\ContactForm7_View', 'render_panel' ),
		);

		return $panels;
	}

	public function insert_property( $properties )
	{
		return wp_parse_args(
			$properties,
			array(
				'followize'               => array(),
				'followize-api'           => array(),
				'followize-fields'        => array(),
				'followize-custom-fields' => array(),
			)
		);
	}

	public function save_property( $contact_form )
	{
		$properties = $contact_form->get_properties();

		$properties['followize']               = Utils::get_prop( 'wpcf7-followize' );
		$properties['followize-api']           = Utils::get_prop( 'wpcf7-followize-api' );
		$properties['followize-fields']        = Utils::get_prop( 'wpcf7-followize-fields' );
		$properties['followize-custom-fields'] = Utils::get_prop( 'wpcf7-followize-custom-fields' );

		$contact_form->set_properties( $properties );
	}

	public function add_hub_utmz_hidden()
	{
		return array( 'hubUtmz' => '' );
	}

	public function do_integration( $cf7 )
	{
		$bases = [
			'consent',
			'legitimate_interest',
			'pre_existing_contract',
			'legal_obligation',
			'health_protection',
			'public_interest'
		];

		$api = Utils::get_api_credentials( $cf7 );

		if ( ! $api['clientKey'] ) {
			return;
		}

		$fields = Utils::get_posted_fields( $cf7 );

		if ($fields['processingLegalBase'] && (in_array($fields['processingLegalBase'], $bases))) {
			$processingLegalBase = [
				"processing" => [
					"name" => $fields['processingLegalBase'],
					"optIn" => null
				]
			];

			unset($fields['processingLegalBase']);
		} else {
			$processingLegalBase = [
				"processing" => [
					"name" => null,
					"optIn" => null
				]
			];
		}

		if ($fields['communicationLegalBase'] && (in_array($fields['communicationLegalBase'], $bases))) {
			$communicationLegalBase = [
				"communication" => [
					"name" => $fields['communicationLegalBase'],
					"optIn" => true
				]
			];

			unset($fields['communicationLegalBase']);
		} else {
			$communicationLegalBase = [
				"communication" => [
					"name" => null,
					"optIn" => null
				]
			];
		}

		$api['legalBases'] = $processingLegalBase + $communicationLegalBase;

		$response = wp_remote_post(
			App::API_URL,
			array(
				'timeout'   => 60,
				'body'      => json_encode( $api + $fields ),
				'sslverify' => false,
			)
		);

		if ( ! Utils::get_array_prop( $cf7->prop( 'followize' ), 'log' ) ) {
			return;
		}

		if ( is_wp_error( $response ) ) {
			error_log( __( 'CF7 Followize Extension: a requisição falhou!', App::PLUGIN_SLUG ) );
			return;
		}

		$response = json_decode( $response['body'], true );

		if ( ! @$response['success'] ) {
			error_log(
				sprintf(
					__( 'CF7 Followize Extension: Fields => %s, erro: %s', App::PLUGIN_SLUG ),
					json_encode( $fields ),
					Utils::get_error_string( @$response['error'] )
				)
			);
		}
	}
}
