<?php
namespace Followize\Extension\CF7;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

use WPCF7_Submission;

class Utils
{
	public static function get_fields_tag_value( $fields, $posted_data )
	{
		if ( ! self::is_valid_array( $fields ) ) {
			return array();
		}

		foreach ( $fields as &$field ) {
			$name  = preg_replace( '/\[\s*([a-zA-Z_][0-9a-zA-Z:._-]*)\s*\]/', '$1', $field );
			$field = self::array_to_string( self::get_array_prop( $posted_data, $name ) );
		}

		return array_filter( $fields );
	}

	public static function get_custom_fields_tag_value( $fields, $posted_data )
	{
		$list = array();

		if ( ! self::is_valid_array( $fields ) ) {
			return $list;
		}

		foreach ( $fields as $field ) {
			if ( ! isset( $field['value'] ) || ! isset( $field['label'] ) ) {
				continue;
			}

			$name                    = preg_replace( '/\[\s*([a-zA-Z_][0-9a-zA-Z:._-]*)\s*\]/', '$1', $field['value'] );
			$list[ $field['label'] ] = self::array_to_string( self::get_array_prop( $posted_data, $name ) );
		}

		return array_filter( $list );
	}

	public static function get_hidden_fields( $posted_data )
	{
		return array( 'hubUtmz' => self::array_to_string( self::get_array_prop( $posted_data, 'hubUtmz' ) ) );
	}

	public static function get_posted_fields( $cf7 )
	{
		$posted_data = WPCF7_Submission::get_instance()->get_posted_data();

		$fields = self::get_fields_tag_value( $cf7->prop( 'followize-fields' ), $posted_data );
		$custom = self::get_custom_fields_tag_value( $cf7->prop( 'followize-custom-fields' ), $posted_data );
		$hidden = self::get_hidden_fields( $posted_data );

		return self::_merge_fields( $fields, $custom, $hidden );
	}

	public static function get_api_fields()
	{
		return array(
			'name' => array(
				'label'       => __( 'Nome do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-name]', App::PLUGIN_SLUG ),
			),
			'email' => array(
				'label'       => __( 'E-mail do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-email]', App::PLUGIN_SLUG ),
			),
			'message' => array(
				'label'       => __( 'Mensagem', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-message]', App::PLUGIN_SLUG ),
			),
			'phone' => array(
				'label'       => __( 'Telefone do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'cellPhone' => array(
				'label'       => __( 'Celular do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'addressLine1' => array(
				'label'       => __( 'Endereço do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'addressLine2' => array(
				'label'       => __( 'Complemento do endereço do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'neighborhood' => array(
				'label'       => __( 'Bairro do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'city' => array(
				'label'       => __( 'Cidade do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'state' => array(
				'label'       => __( 'Estado do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'country' => array(
				'label'       => __( 'País do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'zipCode' => array(
				'label'       => __( 'CEP do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'registrationNumber' => array(
				'label'       => __( 'CPF do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'jobTitle' => array(
				'label'       => __( 'Cargo do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'companyName' => array(
				'label'       => __( 'Nome da empresa do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'companyWebsite' => array(
				'label'       => __( 'Website do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'companyRegistrationNumber' => array(
				'label'       => __( 'CNPJ da empresa do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialTwitter' => array(
				'label'       => __( 'Usuário no Twitter do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialFacebook' => array(
				'label'       => __( 'Usuário no Facebook do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialLinkedin' => array(
				'label'       => __( 'Usuário no LinkedIn do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialGoogleplus' => array(
				'label'       => __( 'Usuário no Google Plus do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialSkype' => array(
				'label'       => __( 'Usuário no Skype do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialWhatsapp' => array(
				'label'       => __( 'Número de telefone cadastrado no Whatsapp do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'socialInstagram' => array(
				'label'       => __( 'Usuário no Instagram do cliente', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'emailOptIn' => array(
				'label'       => __( 'Cliente aceita ou não receber email marketing', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'productId' => array(
				'label'       => __( 'ID do produto cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'productTitle' => array(
				'label'       => __( 'Título do produto', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'productRefer' => array(
				'label'       => __( 'ID de referência do produto cadastrado no site e/ou ERP próprio', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'categoryId' => array(
				'label'       => __( 'ID da categoria cadastrada no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'categoryTitle' => array(
				'label'       => __( 'Título da categoria', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'categoryRefer' => array(
				'label'       => __( 'ID de referência da categoria cadastrada no site e/ou ERP próprio', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'brandId' => array(
				'label'       => __( 'ID da marca cadastrada no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'brandTitle' => array(
				'label'       => __( 'Título da marca', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'brandRefer' => array(
				'label'       => __( 'ID de referência da marca cadastrada no site e/ou ERP próprio', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'locationId' => array(
				'label'       => __( 'ID da unidade cadastrada no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'locationTitle' => array(
				'label'       => __( 'Título da unidade', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'locationRefer' => array(
				'label'       => __( 'ID de referência da unidade cadastrada no site e/ou ERP próprio', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest1Title' => array(
				'label'       => __( 'Título do interesse 1', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest1Id' => array(
				'label'       => __( 'ID do Interesse 1 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest2Title' => array(
				'label'       => __( 'Título do interesse 2', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest2Id' => array(
				'label'       => __( 'ID do Interesse 2 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest3Title' => array(
				'label'       => __( 'Título do interesse 3', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest3Id' => array(
				'label'       => __( 'ID do Interesse 3 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest4Title' => array(
				'label'       => __( 'Título do interesse 4', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest4Id' => array(
				'label'       => __( 'ID do Interesse 4 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest5Title' => array(
				'label'       => __( 'Título do interesse 5', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest5Id' => array(
				'label'       => __( 'ID do Interesse 5 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest6Title' => array(
				'label'       => __( 'Título do interesse 6', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest6Id' => array(
				'label'       => __( 'ID do Interesse 6 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest7Title' => array(
				'label'       => __( 'Título do interesse 7', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'interest7Id' => array(
				'label'       => __( 'ID do Interesse 7 cadastrado no Followize', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'processingLegalBase' => array(
				'label'       => __( 'Base legal de processamento', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
			'communicationLegalBase' => array(
				'label'       => __( 'Base legal de comunicação', App::PLUGIN_SLUG ),
				'placeholder' => __( '[your-tag]', App::PLUGIN_SLUG ),
			),
		);
	}

	public static function get_error_string( $id )
	{
		$errors = array(
			'4000' => __( 'Um ou mais campos obrigatórios não enviados.', App::PLUGIN_SLUG ),
			'4001' => __( 'Chave de cliente inválida.', App::PLUGIN_SLUG ),
			'4002' => __( 'Chave de equipe inválida.', App::PLUGIN_SLUG ),
			'4003' => __( 'Falha ao cadastrar o contato.', App::PLUGIN_SLUG ),
			'4004' => __( 'Nenhum atendente encontrado na equipe enviada.', App::PLUGIN_SLUG ),
			'4005' => __( 'Falha ao cadastrar a conversão.', App::PLUGIN_SLUG ),
			'4006' => __( 'Verificar bases legais.', App::PLUGIN_SLUG ),
			null   => __( 'Erro não tratado.', App::PLUGIN_SLUG ),
		);

		return sprintf( '%s - %s', $id, $errors[ $id ] );
	}

	public static function suggest_field_tags( $cf7 )
	{
		$fields        = (array) $cf7->prop( 'followize-fields' );
		$custom_fields = wp_list_pluck( (array) $cf7->prop( 'followize-custom-fields' ), 'value' );

		$merged_field = array_filter( $fields + $custom_fields );

		foreach ( (array) $cf7->collect_mail_tags() as $tag ) {
			$pattern = sprintf( '/\[(_[a-z]+_)?%s([ \t]+[^]]+)?\]/', preg_quote( $tag, '/' ) );
			$used    = preg_grep( $pattern, $merged_field );

			echo sprintf(
				'<span class="%1$s">[%2$s]</span>',
				'mailtag code ' . ( $used ? 'used' : 'unused' ),
				esc_html( $tag )
			);
		}
	}

	public static function get_api_credentials( $cf7 )
	{
		$api = wp_parse_args(
			$cf7->prop( 'followize-api' ),
			array(
				'clientKey'      => '',
				'teamKey'        => '',
				'conversionGoal' => 'API v2.0',
			)
		);

		$api['teamKey'] = preg_replace( '/.+:\s?/', '', $api['teamKey'] );

		return $api;
	}

	public static function get_prop( $prop )
	{
		if ( ! isset( $_POST[ $prop ] ) ) {
			return array();
		}

		switch ( $prop ) {

			case 'wpcf7-followize-custom-fields' :
				return self::sanitize_sub_array(
					'sanitize_text_field',
					self::filter_custom_fields( $_POST[ $prop ] )
				);

			default :
				return array_map( 'sanitize_text_field', $_POST[ $prop ] );
		}
	}

	public static function filter_custom_fields( $data )
	{
		return array_filter( $data, function( $item ) {
			return $item['value'];
		} );
	}

	public static function sanitize_sub_array( $func, $data )
	{
		foreach ( $data as $key => $value ) {
			foreach ( $value as $k => $v ) {
				$data[ $key ][ $k ] = ( is_callable( $func ) ) ? $func( $v ) : $v;
			}
		}

		return $data;
	}

	public static function get_array_prop( $arr, $prop, $default = '' )
	{
		if ( ! isset( $arr[ $prop ] ) || empty( $arr[ $prop ] ) ) {
			return $default;
		}

		return $arr[ $prop ];
	}

	private static function _merge_fields( $fields, $custom_fields, $hidden )
	{
		$fields = wp_parse_args(
			$fields,
			array( 'message' => '' )
		);

		if ( $custom_fields ) {
			$fields['message'] .= __( "<br />", App::PLUGIN_SLUG ) . self::_format_custom_fields( $custom_fields );
		}

		return ( $fields + $hidden );
	}

	private static function _format_custom_fields( $custom_fields )
	{
		$str = '';

		foreach ( $custom_fields as $key => $value ) {
			$str .= sprintf( '%s: %s <br />', $key, self::array_to_string( $value ) );
		}
		return rtrim( $str, ', ' );
	}

	public static function array_to_string( $fields )
	{
		if ( is_array( $fields ) ) {
			return implode( ', ', array_map( __METHOD__, $fields ) );
		}

		return $fields;
	}

	public static function is_valid_array( $array ) {
		return ( is_array( $array ) && ! empty( $array ) );
	}
}